#include "WAT128.h"



#include <avr/pgmspace.h>



#ifdef _USE_SCI0_

void OpenSCI0(UINT16 iBaud)
{ 
	UINT16 uiBaud = (MAIN_CLK/16/iBaud) -1;
	UCSR0A=0x00;
	UCSR0B=0x98;
	UCSR0C=0x06;
	UBRR0H=uiBaud>>8;
	UBRR0L=uiBaud>>0;

	structRX0Info.rx_wr_index1=0;
	structRX0Info.rx_rd_index1=0;

	structTX0Info.rd_index1=0;
	structTX0Info.wr_index1=0;
	structTX0Info.bComplete = TRUE;



}

ISR(USART0_RX_vect)				/* UART0 interrupt function */
{
	unsigned char status,uart_data;

	status=UCSR0A;
	uart_data = UDR0;			/* Read receive data */
	if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))==0)
	{          
		if (structRX0Info.rx_wr_index1 >= RX_BUFFER_SIZE-1) structRX0Info.rx_wr_index1=0;
		structRX0Info.rx_buffer[structRX0Info.rx_wr_index1]=uart_data;
		structRX0Info.rx_wr_index1++; 
	}
}


INT16S GetByte0(void){
	INT16S iReturnValue = -1;
	cli();                              // global interrupt disable
	if(	structRX0Info.rx_rd_index1 >= structRX0Info.rx_wr_index1){
		structRX0Info.rx_rd_index1 = 0;
		structRX0Info.rx_wr_index1 = 0;
		iReturnValue = -1;
	}else{
		if(structRX0Info.rx_rd_index1 >= RX_BUFFER_SIZE-1){
			structRX0Info.rx_rd_index1 = 0;
		}
		iReturnValue = structRX0Info.rx_buffer[structRX0Info.rx_rd_index1];
		structRX0Info.rx_rd_index1++;
	}
	sei();                              // global interrupt enable
	return iReturnValue;
}


//!    
BOOL PutChar0(char c)
{
	BOOL	bReturn = FALSE;
	UINT16 iTemp=0;
	cli();                              // global interrupt disable
	structTX0Info.bComplete = FALSE;
	iTemp=0;
	while (!( UCSR0A & DATA_REGISTER_EMPTY)){
		if(iTemp++ >1000){
			bReturn =  FALSE;
			break;
		}
	}
	UDR0=c;
	sei();                              // global interrupt enable
	return bReturn;
}

void PutBytes0(BYTE *data, int iLength)
{
	int i=0;
	for(i=0;i<iLength;i++)
	{
		PutChar0(*(data+i));
	}


}

void PutData0(BYTE *data, int iLength)
{
	int i=0;
	BYTE byteCRC = 0;
	PutChar0(0x02);
	for(i=0;i<iLength;i++)
	{
		byteCRC +=*(data+i);
		PutChar0(*(data+i));
	}
	PutChar0(byteCRC);
	
	PutChar0(0x03);

}


//!  ڿ 
BOOL PutString0(char  *string ){
	BYTE i;
	i=0;
	while((string[i])!='\0' ){
		PutChar0(string[i]);
		i++;
	}
	return TRUE;
}


#endif


#ifdef _USE_SCI1_

void InitSCI1(UINT16 iBaud)
{
	UINT16 uiBaud = (MAIN_CLK/16/iBaud) -1;

	UCSR1A=0x00;
	UCSR1B=0x98;
	UCSR1C=0x06;
	UBRR1H=uiBaud>>8;
	UBRR1L=uiBaud>>0;

	structRX1Info.rx_wr_index1=0;
	structRX1Info.rx_rd_index1=0;

	structTX1Info.rd_index1=0;
	structTX1Info.wr_index1=0;
	structTX1Info.bComplete = TRUE;

}

ISR(USART1_RX_vect)				/* UART1 interrupt function */
{
	char status,uart_data;

	status=UCSR1A;
	uart_data = UDR1;			/* Read receive data */
	if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))==0)
	{          
		if (structRX1Info.rx_wr_index1 >= RX_BUFFER_SIZE-1) structRX1Info.rx_wr_index1=0;
		structRX1Info.rx_buffer[structRX1Info.rx_wr_index1]=uart_data;
		structRX1Info.rx_wr_index1++; 

	}
}


INT16S GetByte1(void){
	INT16S iReturnValue = -1;
	cli();                              // global interrupt disable
	if(	structRX1Info.rx_rd_index1 >= structRX1Info.rx_wr_index1){
		structRX1Info.rx_rd_index1 = 0;
		structRX1Info.rx_wr_index1 = 0;
		iReturnValue = -1;
	}else{
		if(structRX1Info.rx_rd_index1 >= RX_BUFFER_SIZE-1){
			structRX1Info.rx_rd_index1 = 0;
		}
		iReturnValue = structRX1Info.rx_buffer[structRX1Info.rx_rd_index1];
		structRX1Info.rx_rd_index1++;
	}
	sei();                              // global interrupt enable
	return iReturnValue;
}


//!    
BOOL PutByte1(unsigned char c)
{
	BOOL	bReturn = FALSE;
	UINT16 iTemp=0;
	cli();                              // global interrupt disable
	structTX1Info.bComplete = FALSE;
	iTemp=0;
	while (!( UCSR1A & DATA_REGISTER_EMPTY)){
		if(iTemp++ >1000){
			bReturn =  FALSE;
			break;
		}
	}
	UDR1=c;
	sei();                              // global interrupt enable
	return bReturn;
}

//!  ڿ 
BOOL PutString1(char  *string ){
	BYTE i;
	i=0;
	while((string[i])!='\0' ){
		PutByte1(string[i]);
		i++;
	}
	return TRUE;
}

#endif


#ifdef _USE_KEY4x4_

void InitKey4x4()
{
	KEY4x4_PORT_DIR = 0x0F;

}

UINT16 GetKey4x4()
{
	UINT16 uiReturnValue  =0;
	
	int i=0;
	// 
	for(i=3;i>=0;i--)
	{
		uiReturnValue <<= 4;
		KEY4x4_PORT = (1<<i);
		DelayUS(1);

		uiReturnValue |= (KEY4x4_PORT_IN>>4);
	}

	return uiReturnValue;
}

void SendKey4x4(UINT16 uidata)
{		
	BYTE byteData[10];
	int iDataLength=0;

	byteData[iDataLength++] = 0x11;
	byteData[iDataLength++] = uidata>>8;
	byteData[iDataLength++] = uidata;
	byteData[iDataLength++] = 0;
	byteData[iDataLength++] = 0;

	PutData0(&byteData,iDataLength);

}

#endif

#ifdef _USE_CLCD_

BYTE CLCD_BusyCheck()
{
	DelayUS(200);
	DelayUS(200);
	DelayUS(200);
	DelayUS(200);
	DelayUS(200);
	DelayUS(200);
	DelayUS(200);


	return (0);

} 


void CLCD_PutString(BYTE x, BYTE y,  char *str)
{
	int len = 0;

	CLCD_XY(x,y);

	while(*str ){
		if(len>16) break;
		CLCD_PutChar(*str++); 
		len++;
	}

}

void CLCD_PutChar(BYTE data)
{
	while(CLCD_BusyCheck());
	ClearBit(CLCD_CONTROL_PORT,CLCD_EA);
	SetBit(CLCD_CONTROL_PORT,CLCD_RS);
	DelayUS(2);
	CLCD_DATA_PORT= data;
	DelayUS(50);
	SetBit(CLCD_CONTROL_PORT,CLCD_EA);
	DelayUS(50);
	ClearBit(CLCD_CONTROL_PORT,CLCD_EA);
}

void CLCD_Clear()
{
	CLCD_Command(1);
	DelayMS(1);
	CLCD_Command(2);
	DelayMS(1);        
}

void CLCD_XY(BYTE x, BYTE y)
{
	BYTE position = 0;
	if(x>16) x = 0;
	switch(y)
	{
	case 0 :
		position = 0x80;
		break;
	case 1 :
		position = 0xC0;
		break;
	}
	CLCD_Command(position + x);
}

void CLCD_Command(BYTE data)
{
	while(CLCD_BusyCheck()){}
	ClearBit(CLCD_CONTROL_PORT,CLCD_RS);
	ClearBit(CLCD_CONTROL_PORT,CLCD_EA);
	DelayUS(1);
	CLCD_DATA_PORT= data;
	DelayUS(50);
	SetBit(CLCD_CONTROL_PORT,CLCD_EA);
	DelayUS(20);
	ClearBit(CLCD_CONTROL_PORT,CLCD_EA);
	DelayUS(50);
} 
 
void CLCD_On()
{
	CLCD_Command(0x01);// dummy
	CLCD_Command(0x38);
	CLCD_Command(0x0C );
	CLCD_Command(0x01);
	CLCD_Command(0x06);
	CLCD_Command(0x01);
	DelayMS(2);
}
   
void CLCD_Init()
{  
	SetBit(CLCD_CONTROL_PORT_DIR,CLCD_EA);
	SetBit(CLCD_CONTROL_PORT_DIR,CLCD_RS);
	SetBit(CLCD_CONTROL_PORT_DIR,CLCD_RW);

	ClearBit(CLCD_CONTROL_PORT,CLCD_RW);
  
	CLCD_DATA_PORT_DIR = 0xFF;

	CLCD_On();
	CLCD_Clear();
}
  

#endif

 


#ifdef _USE_GLCD_
extern prog_uchar   E_font[128][16];

extern prog_uint16_t KSTable[2350] ;

extern  prog_uchar  table_cho[21];
extern prog_uchar  table_joong[30];
extern prog_uchar  table_jong[30];

extern prog_uchar  bul_cho1[22];
extern prog_uchar  bul_cho2[22];
extern prog_uchar  bul_jong[22];

extern  prog_uchar  K_font[360][32] ;




void GLCD_Init()
{

	SetBit(GLCD_CONTROL_PORT_DIR,GLCD_CS1);
	SetBit(GLCD_CONTROL_PORT_DIR,GLCD_CS2);
	SetBit(GLCD_CONTROL_PORT_DIR,GLCD_RST);
	SetBit(GLCD_CONTROL_PORT_DIR,GLCD_RW);
	SetBit(GLCD_CONTROL_PORT_DIR,GLCD_RS);
	SetBit(GLCD_CONTROL_PORT_DIR,GLCD_EA);


	GLCD_DATA_PORT_DIR = 0xFF;

	ClearBit(GLCD_CONTROL_PORT,GLCD_RST);
	DelayUS(10);

	SetBit(GLCD_CONTROL_PORT,GLCD_RST);

	ClearBit(GLCD_CONTROL_PORT,GLCD_RW);


	g_GLCD.pos.x = 0;
	g_GLCD.pos.y = 0;


	g_GLCD.ShowCursor = 0;

	g_GLCD.posCursor.x = 0;
	g_GLCD.posCursor.y = 0;

	CLR_EA;
	CLR_CS1;
	CLR_CS2;
	CLR_RS;

	GLCD_Command(0x00,0x3F);	// CS1, CS2 display ON
	GLCD_Command(0x00,0xC0);	// CS1, CS2 display position


	DelayMS(1);

	GLCD_String(0,0,"                ");
	GLCD_String(1,0,"                ");
	GLCD_String(2,0,"                ");
	GLCD_String(3,0,"                ");

} 




void GLCD_Command(BYTE byteSignal,BYTE byteCommand){

	// E=0, DI = 0;
	CLR_EA;
	CLR_RS;


	if(0x02 == (byteSignal&0x02))
	{
		SET_CS1;
	}
	else
	{
		CLR_CS1;
	}
	if(0x01 == (byteSignal&0x01))
	{
		SET_CS2;
	}
	else
	{
		CLR_CS2;
	}


	DelayUS(1);
	GLCD_DATA_PORT = byteCommand;


	SET_EA;
	CLR_EA;



	SET_CS1;
	SET_CS2;
	DelayUS(1);


}

// x,y ǥ ڰ   ִ.
void GLCD_SetXY(BYTE x, BYTE y)
{
	g_GLCD.pos.x = x;
	g_GLCD.pos.y = y;

}


//   ̴ 16bit  /Ʒ  ؾ Ѵ.
//  ½ /Ʒ
// x : x position
// y : y position
// row  0: upper
//		1: lower
void GLCD_SetXY_ROW(struct _POSITION_ pos,BYTE upperrow)
{
	if(upperrow == 0)					//   Ͱ ̸
		GLCD_Command(0x00,0xB8 + pos.x*2);
	else								//   Ͱ Ʒ̸
		GLCD_Command(0x00,0xB8 + pos.x*2 + 1);

	if(pos.y <= 7)					//  LCD ̸
		GLCD_Command(0x01,0x40 + pos.y*8);
	else							//  LCD ̸
		GLCD_Command(0x02,0x40 + (pos.y-8)*8);
}


void GLCD_WriteData(BYTE byteSignal,BYTE byteChar,BYTE bBlock){

	if(1 == bBlock)
	{
		byteChar = ~byteChar;
	}

	CLR_EA;
	SET_RS;


	if(0x02 == (byteSignal&0x02))
	{
		SET_CS1;
	}
	else
	{
		CLR_CS1;
	}
	if(0x01 == (byteSignal&0x01))
	{
		SET_CS2;
	}
	else
	{
		CLR_CS2;
	}


	DelayUS(1);

	GLCD_DATA_PORT = byteChar;				

	SET_EA;
	CLR_EA;

	SET_CS1;
	SET_CS2;

	DelayUS(1);
}



BYTE GLCD_ReadData(BYTE byteSignal)
{
	BYTE byteChar;
	CLR_EA;
	SET_RS;

	SetBit(GLCD_CONTROL_PORT,GLCD_RW);
	GLCD_DATA_PORT_DIR = 0x00;

	if(0x02 == (byteSignal&0x02))
	{
		SET_CS1;
	}
	else
	{
		CLR_CS1;
	}
	if(0x01 == (byteSignal&0x01))
	{
		SET_CS2;
	}
	else
	{
		CLR_CS2;
	}

	DelayUS(1);


	SET_EA;
	CLR_EA;
	byteChar = GLCD_DATA_INPUT;
	GLCD_DATA_PORT_DIR = 0xFF;

	ClearBit(GLCD_CONTROL_PORT,GLCD_RW);


	SET_CS1;
	SET_CS2;

	return byteChar;
}


// 8x16 ũ  
void GLCD_English(BYTE Ecode,BYTE bBlock)	
{ 
	BYTE i, byteCS;

	if(g_GLCD.pos.y <= 7)		//  LCD 	
		byteCS = 0x01;
	else						//  LCD 	
		byteCS = 0x02;

	if(g_GLCD.ShowCursor==1)
	{
		GLCD_RemoveCursor();
	}

	//   ũ 16  8  
	GLCD_SetXY_ROW(g_GLCD.pos,0);		



	for(i = 0; i <= 7; i++)
	{
		GLCD_WriteData(byteCS, pgm_read_byte(&E_font[Ecode][i]),bBlock);
	}



	//    ũ 16 ϴ 8  
	GLCD_SetXY_ROW(g_GLCD.pos,1);	 

	for(i = 8; i <= 15; i++)
	{ 
		if((g_GLCD.ShowCursor == 1))
		{
			// Ŀ  Ѵٸ
			g_GLCD.posCursor = g_GLCD.pos;
			GLCD_WriteData(byteCS,pgm_read_byte(&E_font[Ecode][i]) | 0x80,bBlock);
		}
		else
			GLCD_WriteData(byteCS,pgm_read_byte(&E_font[Ecode][i]),bBlock);
	}


	g_GLCD.pos.y++;                              
	if(g_GLCD.pos.y == 16) 
	{
		g_GLCD.pos.y = 0;
		if(++g_GLCD.pos.x >=4) g_GLCD.pos.x=0;
	}
}

// 16x16 ũ ѱ 
void GLCD_Korean(UINT16 Kcode,BYTE byteBlock)
{ 
	BYTE i, byteCS;
	BYTE cho_5bit, joong_5bit, jong_5bit;
	BYTE cho_bul, joong_bul, jong_bul=0, jong_flag;
	unsigned int character;
	BYTE Korean_buffer[32];	// ӽ 

	if(g_GLCD.pos.y == 15)				//   1ĭ̶  ϰ .
		GLCD_English(0x20,byteBlock);


	if(g_GLCD.pos.y <= 7)		//  LCD 	
		byteCS = 0x01;
	else						//  LCD 	
		byteCS = 0x02;



	// ϴ ʼ/߼/ 
	cho_5bit =  table_cho[(Kcode >> 10) & 0x001F];
	joong_5bit =  table_joong[(Kcode >> 5) & 0x001F];
	jong_5bit =  table_jong[Kcode & 0x001F];

	if(jong_5bit == 0)				//   
	{
		jong_flag = 0;
		cho_bul = bul_cho1[joong_5bit];
		if((cho_5bit == 1) || (cho_5bit == 16)) joong_bul = 0;
		else                                    joong_bul = 1;
	}
	else						//  ִٸ
	{
		jong_flag = 1;
		cho_bul = bul_cho2[joong_5bit];
		if((cho_5bit == 1) || (cho_5bit == 16)) joong_bul = 2;
		else                                    joong_bul = 3;
		jong_bul = bul_jong[joong_5bit];
	}


	// ʼ   
	character = (UINT16)cho_bul*20 + (UINT16)cho_5bit;		 
	for(i = 0; i <= 31; i++)
		Korean_buffer[i] = pgm_read_byte(&K_font[character][i]);


	// ߼  ͼ ġ
	character = (UINT16)8*20 + (UINT16)joong_bul*22 + (UINT16)joong_5bit;
	for(i = 0; i <= 31; i++)
		Korean_buffer[i] |= pgm_read_byte(&K_font[character][i]);

	//    ġ
	if(jong_flag == 1)		
	{
		character = (UINT16)8*20 + (UINT16)4*22 +(UINT16)jong_bul*28 + (UINT16)jong_5bit;
		for(i = 0; i <= 31; i++)
			Korean_buffer[i] |= pgm_read_byte(&K_font[character][i]);
	}



	//  ߰ 
	// 	if(TRUE == byteBlock){
	// 		for(i = 0; i <= 31; i++)
	// 			Korean_buffer[i] = ~Korean_buffer[i];  
	//   	}

	// 16 x16 ũ⿡   
	GLCD_SetXY_ROW(g_GLCD.pos,0);		//  8 
	for(i = 0; i <= 7; i++)
		GLCD_WriteData(byteCS,Korean_buffer[i],byteBlock);

	// 16 x16 ũ⿡  ϴ 
	GLCD_SetXY_ROW(g_GLCD.pos,1);		 
	for(i = 16; i <= 23; i++)
	{ 
		if(g_GLCD.ShowCursor == 1) 
			GLCD_WriteData(byteCS,Korean_buffer[i] | 0x80,byteBlock);
		else
			GLCD_WriteData(byteCS,Korean_buffer[i],byteBlock);
	}






	g_GLCD.pos.y++;

	// 16 x16 ũ⿡   
	GLCD_SetXY_ROW(g_GLCD.pos,0);		 
	for(i = 8; i <= 15; i++)
		GLCD_WriteData(byteCS,Korean_buffer[i],byteBlock);


	// 16 x16 ũ⿡  ϴ 
	GLCD_SetXY_ROW(g_GLCD.pos,1);		
	for(i = 24; i <= 31; i++)
	{ 
		if(g_GLCD.ShowCursor == 1) 
			GLCD_WriteData(byteCS,Korean_buffer[i] | 0x80,byteBlock);
		else
			GLCD_WriteData(byteCS,Korean_buffer[i],byteBlock);
	}

	g_GLCD.pos.y++;                         
	if(g_GLCD.pos.y == 16)
	{
		g_GLCD.pos.y = 0;
		g_GLCD.pos.x++;
	}
}




void GLCD_String(BYTE x,BYTE y,BYTE const *string)	
{
	BYTE character1;
	unsigned int  character2;


	g_GLCD.pos.x = x;
	g_GLCD.pos.y = y;	


	while(*string != '\0')
	{
		character1 = *string;
		string++;
		if(character1 < 0x80)
			GLCD_English(character1,FALSE);		
		else
		{
			character2 = 256*character1 + (*string & 0xFF);
			string++;
			GLCD_Korean(character2,0);	
		}     
	}
}


void GLCD_ShowCursor(UINT8 _show)
{
	g_GLCD.ShowCursor = _show;
}


void GLCD_RemoveCursor()
{
	BYTE byteData[8];
	BYTE byteCS=0;

	if(g_GLCD.ShowCursor == 0)
		return;

	GLCD_SetXY_ROW(g_GLCD.posCursor,1);

	if(g_GLCD.posCursor.y <= 7)	//  ȭ̸
		byteCS = 0x01;
	else					 
		byteCS = 0x02;



	GLCD_ReadData(byteCS);
	byteData[0] = GLCD_ReadData(byteCS);
	if(byteData[0] & 0x80 )
	{
		// Ŀ black ̸
		byteData[0] &= 0x7F;
		byteData[1] = GLCD_ReadData(byteCS) & 0x7F;
		byteData[2] = GLCD_ReadData(byteCS) & 0x7F;
		byteData[3] = GLCD_ReadData(byteCS) & 0x7F;
		byteData[4] = GLCD_ReadData(byteCS) & 0x7F;
		byteData[5] = GLCD_ReadData(byteCS) & 0x7F;
		byteData[6] = GLCD_ReadData(byteCS) & 0x7F;
		byteData[7] = GLCD_ReadData(byteCS) & 0x7F;	}
	else
	{
		// Ŀż white̸ (ַ    )
		byteData[0] |= 0x80;
		byteData[1] = GLCD_ReadData(byteCS) | 0x80;
		byteData[2] = GLCD_ReadData(byteCS)  | 0x80;;
		byteData[3] = GLCD_ReadData(byteCS)  | 0x80;
		byteData[4] = GLCD_ReadData(byteCS)  | 0x80;
		byteData[5] = GLCD_ReadData(byteCS)  | 0x80;;
		byteData[6] = GLCD_ReadData(byteCS)  | 0x80;
		byteData[7] = GLCD_ReadData(byteCS)  | 0x80;
	}




	GLCD_SetXY_ROW(g_GLCD.posCursor,1);
	GLCD_WriteData(byteCS,byteData[0],0);
	GLCD_WriteData(byteCS,byteData[1],0);
	GLCD_WriteData(byteCS,byteData[2],0);
	GLCD_WriteData(byteCS,byteData[3],0);
	GLCD_WriteData(byteCS,byteData[4],0);
	GLCD_WriteData(byteCS,byteData[5],0);
	GLCD_WriteData(byteCS,byteData[6],0);
	GLCD_WriteData(byteCS,byteData[7],0);

}


void GLCD_DrawRect(int x1,int y1,int x2,int y2)
{

}


#endif


#ifdef _USE_ADC_
void InitADC()
{



		ACSR=0x00;
		SFIOR=0x00;

		ADCSRA=0x85 ;
		ADMUX=ADC_VREF_TYPE;
		DelayUS(250);
		DelayUS(250);
		ADCSRA = 0xD7;
	
}



#endif 



#ifdef _USE_FND4_





void InitFND4()
{
	g_FND[0]=1;
	g_FND[1]=2;
	g_FND[2]=3;
	g_FND[3]=4;


	// FND DATA   (FND 4401ASR2 )
	FND4_DISPLAY_DATA[0] = 		0b00010100;//,		//0
	FND4_DISPLAY_DATA[1] = 			0b11010111;//,		//1
	FND4_DISPLAY_DATA[2] = 			0b01001100;//,		//2
	FND4_DISPLAY_DATA[3] = 			0b01000101;//,		//3
	FND4_DISPLAY_DATA[4] = 			0b10000111;//,		//4
	FND4_DISPLAY_DATA[5] = 			0b00100101;//,		//5
	FND4_DISPLAY_DATA[6] = 			0b00100100;//,		//6
	FND4_DISPLAY_DATA[7] = 			0b01010111;//,		//7
	FND4_DISPLAY_DATA[8] = 			0b00000100;//,		//8
	FND4_DISPLAY_DATA[9] = 			0b00000101;//		//9
	


	FND4_CONTROL_DIR |= 0xF0;
	FND4_DATA_DIR = 0xFF;

}





void DisplayFND4(BYTE b1,BYTE b2,BYTE b3,BYTE b4)
{

	SetBit(FND4_CONTROL_PORT,FND4_SEL1_BIT);
	FND4_DATA = FND4_DISPLAY_DATA[b1];
	DelayMS(2);
	ClearBit(FND4_CONTROL_PORT,FND4_SEL1_BIT);


	SetBit(FND4_CONTROL_PORT,FND4_SEL2_BIT);
	FND4_DATA = FND4_DISPLAY_DATA[b2];
	DelayMS(2);
	ClearBit(FND4_CONTROL_PORT,FND4_SEL2_BIT);



	SetBit(FND4_CONTROL_PORT,FND4_SEL3_BIT);
	FND4_DATA = FND4_DISPLAY_DATA[b3];
	DelayMS(2);
	ClearBit(FND4_CONTROL_PORT,FND4_SEL3_BIT);



	SetBit(FND4_CONTROL_PORT,FND4_SEL4_BIT);
	FND4_DATA = FND4_DISPLAY_DATA[b4];
	DelayMS(2);
	ClearBit(FND4_CONTROL_PORT,FND4_SEL4_BIT);

}

#endif

#ifdef _USE_ROTARY_

void InitRotary()
{
	ROTARY_DATA_DIR  &= 0xF0;

}

BYTE GetRotaryInt()
{
	return (~ROTARY_DATA_PORT & 0x0F);
}

#endif



void DelayUS(BYTE time_us)		/* time delay for us */
{ 
	register BYTE i;
	for(i = 0; i < time_us; i++)			// 4 cycle +
	{
		asm volatile(" PUSH  R0 ");		// 2 cycle +
		asm volatile(" POP   R0 ");		// 2 cycle +
		asm volatile(" PUSH  R0 ");		// 2 cycle +
		asm volatile(" POP   R0 ");		// 2 cycle +
		asm volatile(" PUSH  R0 ");		// 2 cycle +
		asm volatile(" POP   R0 ");		// 2 cycle = 16 cycle = 1 us for 16MHz
	}
}

void DelayMS(unsigned int time_ms)		/* time delay for ms */
{ 
	register unsigned int i;
	for(i = 0; i < time_ms; i++)
	{ 
		DelayUS(250);
		DelayUS(250);
		DelayUS(250);
		DelayUS(250);
	}
}